var require = function (file, cwd) {
    var resolved = require.resolve(file, cwd || '/');
    var mod = require.modules[resolved];
    if (!mod) throw new Error(
        'Failed to resolve module ' + file + ', tried ' + resolved
    );
    var res = mod._cached ? mod._cached : mod();
    return res;
}

require.paths = [];
require.modules = {};
require.extensions = [".js",".coffee",".js"];

require._core = {
    'assert': true,
    'events': true,
    'fs': true,
    'path': true,
    'vm': true
};

require.resolve = (function () {
    return function (x, cwd) {
        if (!cwd) cwd = '/';
        
        if (require._core[x]) return x;
        var path = require.modules.path();
        cwd = path.resolve('/', cwd);
        var y = cwd || '/';
        
        if (x.match(/^(?:\.\.?\/|\/)/)) {
            var m = loadAsFileSync(path.resolve(y, x))
                || loadAsDirectorySync(path.resolve(y, x));
            if (m) return m;
        }
        
        var n = loadNodeModulesSync(x, y);
        if (n) return n;
        
        throw new Error("Cannot find module '" + x + "'");
        
        function loadAsFileSync (x) {
            x = path.normalize(x);
            if (require.modules[x]) {
                return x;
            }
            
            for (var i = 0; i < require.extensions.length; i++) {
                var ext = require.extensions[i];
                if (require.modules[x + ext]) return x + ext;
            }
        }
        
        function loadAsDirectorySync (x) {
            x = x.replace(/\/+$/, '');
            var pkgfile = path.normalize(x + '/package.json');
            if (require.modules[pkgfile]) {
                var pkg = require.modules[pkgfile]();
                var b = pkg.browserify;
                if (typeof b === 'object' && b.main) {
                    var m = loadAsFileSync(path.resolve(x, b.main));
                    if (m) return m;
                }
                else if (typeof b === 'string') {
                    var m = loadAsFileSync(path.resolve(x, b));
                    if (m) return m;
                }
                else if (pkg.main) {
                    var m = loadAsFileSync(path.resolve(x, pkg.main));
                    if (m) return m;
                }
            }
            
            return loadAsFileSync(x + '/index');
        }
        
        function loadNodeModulesSync (x, start) {
            var dirs = nodeModulesPathsSync(start);
            for (var i = 0; i < dirs.length; i++) {
                var dir = dirs[i];
                var m = loadAsFileSync(dir + '/' + x);
                if (m) return m;
                var n = loadAsDirectorySync(dir + '/' + x);
                if (n) return n;
            }
            
            var m = loadAsFileSync(x);
            if (m) return m;
        }
        
        function nodeModulesPathsSync (start) {
            var parts;
            if (start === '/') parts = [ '' ];
            else parts = path.normalize(start).split('/');
            
            var dirs = [];
            for (var i = parts.length - 1; i >= 0; i--) {
                if (parts[i] === 'node_modules') continue;
                var dir = parts.slice(0, i + 1).join('/') + '/node_modules';
                dirs.push(dir);
            }
            
            return dirs;
        }
    };
})();

require.alias = function (from, to) {
    var path = require.modules.path();
    var res = null;
    try {
        res = require.resolve(from + '/package.json', '/');
    }
    catch (err) {
        res = require.resolve(from, '/');
    }
    var basedir = path.dirname(res);
    
    var keys = (Object.keys || function (obj) {
        var res = [];
        for (var key in obj) res.push(key)
        return res;
    })(require.modules);
    
    for (var i = 0; i < keys.length; i++) {
        var key = keys[i];
        if (key.slice(0, basedir.length + 1) === basedir + '/') {
            var f = key.slice(basedir.length);
            require.modules[to + f] = require.modules[basedir + f];
        }
        else if (key === basedir) {
            require.modules[to] = require.modules[basedir];
        }
    }
};

(function () {
    var process = {};
    
    require.define = function (filename, fn) {
        if (require.modules.__browserify_process) {
            process = require.modules.__browserify_process();
        }
        
        var dirname = require._core[filename]
            ? ''
            : require.modules.path().dirname(filename)
        ;
        
        var require_ = function (file) {
            return require(file, dirname)
        };
        require_.resolve = function (name) {
            return require.resolve(name, dirname);
        };
        require_.modules = require.modules;
        require_.define = require.define;
        var module_ = { exports : {} };
        
        require.modules[filename] = function () {
            require.modules[filename]._cached = module_.exports;
            fn.call(
                module_.exports,
                require_,
                module_,
                module_.exports,
                dirname,
                filename,
                process
            );
            require.modules[filename]._cached = module_.exports;
            return module_.exports;
        };
    };
})();


require.define("path",Function(['require','module','exports','__dirname','__filename','process'],"function filter (xs, fn) {\n    var res = [];\n    for (var i = 0; i < xs.length; i++) {\n        if (fn(xs[i], i, xs)) res.push(xs[i]);\n    }\n    return res;\n}\n\n// resolves . and .. elements in a path array with directory names there\n// must be no slashes, empty elements, or device names (c:\\) in the array\n// (so also no leading and trailing slashes - it does not distinguish\n// relative and absolute paths)\nfunction normalizeArray(parts, allowAboveRoot) {\n  // if the path tries to go above the root, `up` ends up > 0\n  var up = 0;\n  for (var i = parts.length; i >= 0; i--) {\n    var last = parts[i];\n    if (last == '.') {\n      parts.splice(i, 1);\n    } else if (last === '..') {\n      parts.splice(i, 1);\n      up++;\n    } else if (up) {\n      parts.splice(i, 1);\n      up--;\n    }\n  }\n\n  // if the path is allowed to go above the root, restore leading ..s\n  if (allowAboveRoot) {\n    for (; up--; up) {\n      parts.unshift('..');\n    }\n  }\n\n  return parts;\n}\n\n// Regex to split a filename into [*, dir, basename, ext]\n// posix version\nvar splitPathRe = /^(.+\\/(?!$)|\\/)?((?:.+?)?(\\.[^.]*)?)$/;\n\n// path.resolve([from ...], to)\n// posix version\nexports.resolve = function() {\nvar resolvedPath = '',\n    resolvedAbsolute = false;\n\nfor (var i = arguments.length; i >= -1 && !resolvedAbsolute; i--) {\n  var path = (i >= 0)\n      ? arguments[i]\n      : process.cwd();\n\n  // Skip empty and invalid entries\n  if (typeof path !== 'string' || !path) {\n    continue;\n  }\n\n  resolvedPath = path + '/' + resolvedPath;\n  resolvedAbsolute = path.charAt(0) === '/';\n}\n\n// At this point the path should be resolved to a full absolute path, but\n// handle relative paths to be safe (might happen when process.cwd() fails)\n\n// Normalize the path\nresolvedPath = normalizeArray(filter(resolvedPath.split('/'), function(p) {\n    return !!p;\n  }), !resolvedAbsolute).join('/');\n\n  return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';\n};\n\n// path.normalize(path)\n// posix version\nexports.normalize = function(path) {\nvar isAbsolute = path.charAt(0) === '/',\n    trailingSlash = path.slice(-1) === '/';\n\n// Normalize the path\npath = normalizeArray(filter(path.split('/'), function(p) {\n    return !!p;\n  }), !isAbsolute).join('/');\n\n  if (!path && !isAbsolute) {\n    path = '.';\n  }\n  if (path && trailingSlash) {\n    path += '/';\n  }\n  \n  return (isAbsolute ? '/' : '') + path;\n};\n\n\n// posix version\nexports.join = function() {\n  var paths = Array.prototype.slice.call(arguments, 0);\n  return exports.normalize(filter(paths, function(p, index) {\n    return p && typeof p === 'string';\n  }).join('/'));\n};\n\n\nexports.dirname = function(path) {\n  var dir = splitPathRe.exec(path)[1] || '';\n  var isWindows = false;\n  if (!dir) {\n    // No dirname\n    return '.';\n  } else if (dir.length === 1 ||\n      (isWindows && dir.length <= 3 && dir.charAt(1) === ':')) {\n    // It is just a slash or a drive letter with a slash\n    return dir;\n  } else {\n    // It is a full dirname, strip trailing slash\n    return dir.substring(0, dir.length - 1);\n  }\n};\n\n\nexports.basename = function(path, ext) {\n  var f = splitPathRe.exec(path)[2] || '';\n  // TODO: make this comparison case-insensitive on windows?\n  if (ext && f.substr(-1 * ext.length) === ext) {\n    f = f.substr(0, f.length - ext.length);\n  }\n  return f;\n};\n\n\nexports.extname = function(path) {\n  return splitPathRe.exec(path)[3] || '';\n};\n\n//@ sourceURL=path"));

require.define("__browserify_process",Function(['require','module','exports','__dirname','__filename','process'],"var process = module.exports = {};\n\nprocess.nextTick = (function () {\n    var queue = [];\n    var canPost = typeof window !== 'undefined'\n        && window.postMessage && window.addEventListener\n    ;\n    \n    if (canPost) {\n        window.addEventListener('message', function (ev) {\n            if (ev.source === window && ev.data === 'browserify-tick') {\n                ev.stopPropagation();\n                if (queue.length > 0) {\n                    var fn = queue.shift();\n                    fn();\n                }\n            }\n        }, true);\n    }\n    \n    return function (fn) {\n        if (canPost) {\n            queue.push(fn);\n            window.postMessage('browserify-tick', '*');\n        }\n        else setTimeout(fn, 0);\n    };\n})();\n\nprocess.title = 'browser';\nprocess.browser = true;\nprocess.env = {};\nprocess.argv = [];\n\nprocess.binding = function (name) {\n    if (name === 'evals') return (require)('vm')\n    else throw new Error('No such module. (Possibly not yet loaded)')\n};\n\n(function () {\n    var cwd = '/';\n    var path;\n    process.cwd = function () { return cwd };\n    process.chdir = function (dir) {\n        if (!path) path = require('path');\n        cwd = path.resolve(dir, cwd);\n    };\n})();\n\n//@ sourceURL=__browserify_process"));

require.define("/test/testModules/moduleA.js",Function(['require','module','exports','__dirname','__filename','process']," \"use strict\"; /* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n\"use strict\"; // run code in ES5 strict mode\r\n\r\nvar someOtherModule = require(\"./someOtherModule.js\"),\r\n    myNumber = 0,   // copy by value\r\n    myObj = {},     // copy by reference\r\n    env = \"bla\",\r\n    fs = require(\"fs\");\r\n\r\n// We need getters and setters for private vars to check if our injected setters and getters actual work\r\nfunction setMyNumber(newNumber) {\r\n    myNumber = newNumber;\r\n}\r\n\r\nfunction getMyNumber() {\r\n    return myNumber;\r\n}\r\n\r\nfunction setMyObj(newObj) {\r\n    myObj = newObj;\r\n}\r\n\r\nfunction getMyObj() {\r\n    return myObj;\r\n}\r\n\r\nfunction readFileSync() {\r\n    fs.readFileSync(\"bla.txt\", \"utf8\");\r\n}\r\n\r\nfunction checkSomeGlobals() {\r\n    if (typeof global === \"undefined\") {\r\n        throw new ReferenceError(\"global is undefined\");\r\n    }\r\n    if (typeof console === \"undefined\") {\r\n        throw new ReferenceError(\"console is undefined\");\r\n    }\r\n    if (typeof __filename === \"undefined\") {\r\n        throw new ReferenceError(\"__filename is undefined\");\r\n    }\r\n    if (typeof __dirname === \"undefined\") {\r\n        throw new ReferenceError(\"__dirname is undefined\");\r\n    }\r\n    if (typeof setTimeout === \"undefined\") {\r\n        throw new ReferenceError(\"setTimeout is undefined\");\r\n    }\r\n}\r\n\r\nfunction getConsole() {\r\n    return console;\r\n}\r\n\r\nfunction getFilename() {\r\n    return __filename;\r\n}\r\n\r\n// different styles of exports in moduleA.js and moduleB.js\r\nexports.setMyNumber = setMyNumber;\r\nexports.getMyNumber = getMyNumber;\r\nexports.setMyObj = setMyObj;\r\nexports.getMyObj = getMyObj;\r\nexports.readFileSync = readFileSync;\r\nexports.checkSomeGlobals = checkSomeGlobals;\r\nexports.getConsole = getConsole;\r\nexports.getFilename = getFilename;\r\nexports.someOtherModule = someOtherModule;\n//@ sourceURL=/test/testModules/moduleA.js"));

require.define("/node_modules/rewire/package.json",Function(['require','module','exports','__dirname','__filename','process'],"module.exports = {\"main\":\"../../lib/index.js\"}\n//@ sourceURL=/node_modules/rewire/package.json"));

require.define("/lib/index.js",Function(['require','module','exports','__dirname','__filename','process'],"var rewireModule;\r\n\r\n/**\r\n * Adds a special setter and getter to the module located at filename. After the module has been rewired, you can\r\n * call myModule.__set__(name, value) and myModule.__get__(name) to manipulate private variables.\r\n *\r\n * @param {!String} filename Path to the module that shall be rewired. Use it exactly like require().\r\n * @param {Boolean} cache Indicates whether the rewired module should be cached by node so subsequent calls of require() will return the rewired module. Subsequent calls of rewire() will always overwrite the cache.\r\n * @return {*} the rewired module\r\n */\r\nfunction rewire(filename, cache) {\r\n    if (cache === undefined) {\r\n        cache = true;\r\n    }\r\n\r\n    return rewireModule(module.parent, filename, cache);\r\n}\r\n\r\n// Conditional require for different environments\r\nif (process.title === \"browser\") {\r\n    module.exports = require(\"./browserify/browserifyRewire.js\");\r\n} else {\r\n    delete require.cache[__filename];   // deleting self from module cache so the parent module is always up to date\r\n\r\n    // Putting (require) within brackets is a hack to disable browserify's require sniffing\r\n    // @see https://github.com/substack/node-browserify/issues/132#issuecomment-5281470\r\n    rewireModule = (require)(\"./internalRewire.js\");\r\n\r\n    rewire.reset = rewireModule.reset;\r\n    rewire.browserify = (require)(\"./browserify/browserifyMiddleware.js\");\r\n\r\n    module.exports = rewire;\r\n}\r\n\n//@ sourceURL=/lib/index.js"));

require.define("/lib/browserify/browserifyRewire.js",Function(['require','module','exports','__dirname','__filename','process'],"var pathUtil = require(\"path\"),\r\n    browserifyRequire = window.browserifyRequire;\r\n\r\n// Saves all setters and getters for every module according to its filename\r\nvar registry = {},\r\n// Cache for all rewired modules so it can be reset anytime\r\n    rewiredModules = [];\r\n\r\n/**\r\n * Executes the given module and adds a special setter and getter that allow you to set and get private variables.\r\n * The parentModulePath is usually set by the requireProxy.\r\n *\r\n * @param {!String} parentModulePath __filename of the module, that wants to rewire() another module.\r\n * @param {!String} targetPath path to the module that shall be rewired\r\n * @param {Boolean=true} cache indicates whether the rewired module should be cached or not\r\n * @return {Object}\r\n */\r\nfunction browserifyRewire(parentModulePath, targetPath, cache) {\r\n    var originalModule,\r\n        rewiredModule = {},\r\n        registeredTargetModule;\r\n\r\n    // Default cache to true\r\n    if (cache === undefined) {\r\n        cache = true;\r\n    }\r\n\r\n    // Make absolute paths\r\n    if (targetPath.charAt(0) === \".\") {\r\n        targetPath = pathUtil.resolve(pathUtil.dirname(parentModulePath), targetPath);\r\n    }\r\n\r\n     // Normalize path with file extensions\r\n    targetPath = require.resolve(targetPath);\r\n\r\n    // Deleting module from cache to trigger execution again\r\n    delete browserifyRequire.modules[targetPath]._cached;\r\n\r\n    // Require module to trigger rewire.register() if it hasnt been required yet\r\n    // Putting (require) within brackets is a hack to disable browserifys require sniffing\r\n    // @see https://github.com/substack/node-browserify/issues/132#issuecomment-5281470\r\n    originalModule = (require)(targetPath);\r\n\r\n    // Copy all exported values to our rewired module\r\n    for (var key in originalModule) {\r\n        if (originalModule.hasOwnProperty(key)) {\r\n            rewiredModule[key] = originalModule[key];\r\n        }\r\n    }\r\n\r\n    // If caching is enabled we store the rewiredModule in the cache\r\n    if (cache) {\r\n        browserifyRequire.modules[targetPath]._cached = rewiredModule;\r\n    }\r\n\r\n    // Get registry entry of the target module\r\n    registeredTargetModule = registry[targetPath];\r\n\r\n    // Apply setter and getters\r\n    rewiredModule.__set__ = registeredTargetModule.setter;\r\n    rewiredModule.__get__ = registeredTargetModule.getter;\r\n\r\n    // Store rewired modules for rewire.reset()\r\n    rewiredModules.push(targetPath);\r\n\r\n    return rewiredModule;\r\n}\r\n\r\n/**\r\n * Registers the setter and getter of every module according to its filename\r\n *\r\n * @param {!String} filename the absolute path to the module (module id)\r\n * @param {!Function} setter\r\n * @param {!Function} getter\r\n */\r\nbrowserifyRewire.register = function (filename, setter, getter) {\r\n    registry[filename] = {\r\n        setter: setter,\r\n        getter: getter\r\n    };\r\n};\r\n\r\n/**\r\n * Deletes all rewired modules from the cache\r\n */\r\nbrowserifyRewire.reset = function () {\r\n    var modules = browserifyRequire.modules,\r\n        i;\r\n\r\n    for (i = 0; i < rewiredModules.length; i++) {\r\n        delete modules[rewiredModules[i]]._cached;\r\n    }\r\n\r\n    rewiredModules = [];\r\n};\r\n\r\nmodule.exports = browserifyRewire;\n//@ sourceURL=/lib/browserify/browserifyRewire.js"));

require.define("/test/testModules/someOtherModule.js",Function(['require','module','exports','__dirname','__filename','process']," \"use strict\"; /* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n\"use strict\"; // run code in ES5 strict mode\r\n\r\nvar fs = require(\"fs\");\r\n\r\nexports.fs = fs;\n//@ sourceURL=/test/testModules/someOtherModule.js"));

require.define("fs",Function(['require','module','exports','__dirname','__filename','process'],"/* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n// nothing to see here... no file methods for the browser\n\n//@ sourceURL=fs"));

require.define("/test/testModules/moduleB.js",Function(['require','module','exports','__dirname','__filename','process']," \"use strict\"; /* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n\"use strict\"; // run code in ES5 strict mode\r\n\r\nvar someOtherModule = require(\"./someOtherModule.js\"),\r\n    myNumber = 0,   // copy by value\r\n    myObj = {},     // copy by reference\r\n    env = \"bla\",\r\n    fs = require(\"fs\");\r\n\r\n// We need getters and setters for private vars to check if our injected setters and getters actual work\r\nfunction setMyNumber(newNumber) {\r\n    myNumber = newNumber;\r\n}\r\n\r\nfunction getMyNumber() {\r\n    return myNumber;\r\n}\r\n\r\nfunction setMyObj(newObj) {\r\n    myObj = newObj;\r\n}\r\n\r\nfunction getMyObj() {\r\n    return myObj;\r\n}\r\n\r\nfunction readFileSync() {\r\n    fs.readFileSync(\"bla.txt\", \"utf8\");\r\n}\r\n\r\nfunction checkSomeGlobals() {\r\n    if (typeof global === \"undefined\") {\r\n        throw new ReferenceError(\"global is undefined\");\r\n    }\r\n    if (typeof console === \"undefined\") {\r\n        throw new ReferenceError(\"console is undefined\");\r\n    }\r\n    if (typeof __filename === \"undefined\") {\r\n        throw new ReferenceError(\"__filename is undefined\");\r\n    }\r\n    if (typeof __dirname === \"undefined\") {\r\n        throw new ReferenceError(\"__dirname is undefined\");\r\n    }\r\n    if (typeof setTimeout === \"undefined\") {\r\n        throw new ReferenceError(\"setTimeout is undefined\");\r\n    }\r\n}\r\n\r\nfunction getConsole() {\r\n    return console;\r\n}\r\n\r\nfunction getFilename() {\r\n    return __filename;\r\n}\r\n\r\n// different styles of exports in moduleA.js and moduleB.js\r\nexports.setMyNumber = setMyNumber;\r\nexports.getMyNumber = getMyNumber;\r\nexports.setMyObj = setMyObj;\r\nexports.getMyObj = getMyObj;\r\nexports.readFileSync = readFileSync;\r\nexports.checkSomeGlobals = checkSomeGlobals;\r\nexports.getConsole = getConsole;\r\nexports.getFilename = getFilename;\r\nexports.someOtherModule = someOtherModule;\n//@ sourceURL=/test/testModules/moduleB.js"));

require.define("/test/testModules/emptyModule.js",Function(['require','module','exports','__dirname','__filename','process']," \"use strict\"; /* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n\"use strict\"; // run code in ES5 strict mode\r\n\n//@ sourceURL=/test/testModules/emptyModule.js"));

require.define("/test/testModules/strictModule.js",Function(['require','module','exports','__dirname','__filename','process']," \"use strict\"; /* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n\"use strict\"; // run code in ES5 strict mode\r\n\r\nfunction doSomethingUnstrict() {\r\n    var caller = arguments.callee.caller;   // this should throw an error as a proof that strict mode is on\r\n}\r\n\r\nexports.doSomethingUnstrict = doSomethingUnstrict;\n//@ sourceURL=/test/testModules/strictModule.js"));

require.define("/test/testModules/sharedTestCases.js",Function(['require','module','exports','__dirname','__filename','process'],"/* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n// Don't run code in ES5 strict mode.\r\n// In case this module was in strict mode, all other modules called by this would also be strict.\r\n// But when testing if the strict mode is preserved, we must ensure that this module is NOT strict.\r\n\r\nvar path = require(\"path\"),\r\n    expect = require(\"expect.js\"),\r\n    rewire = require(\"rewire\");\r\n\r\nvar testModules = {\r\n        A: path.resolve(__dirname, \"./moduleA.js\"),\r\n        B: path.resolve(__dirname, \"./moduleB.js\"),\r\n        someOtherModule: path.resolve(__dirname, \"./someOtherModule.js\"),\r\n        emptyModule: path.resolve(__dirname, \"./emptyModule.js\"),\r\n        strictModule: path.resolve(__dirname, \"./strictModule.js\")\r\n    };\r\n\r\n\r\nfunction checkForTypeError(err) {\r\n    expect(err.constructor).to.be(TypeError);\r\n}\r\n\r\nfunction cleanRequireCache() {\r\n    var moduleName,\r\n        modulePath;\r\n\r\n    for (moduleName in testModules) {\r\n        if (testModules.hasOwnProperty(moduleName)) {\r\n            modulePath = testModules[moduleName];\r\n            if (typeof window === \"undefined\") {\r\n                delete require.cache[modulePath];\r\n            } else {\r\n                if (typeof window.browserifyRequire.modules[modulePath]._cached === \"object\") {\r\n                    delete window.browserifyRequire.modules[modulePath]._cached;\r\n                }\r\n            }\r\n        }\r\n    }\r\n}\r\n\r\ndescribe(\"rewire \" + (typeof window === \"undefined\"? \"(node.js)\": \"(browser)\"), function () {\r\n    beforeEach(cleanRequireCache);  // ensuring a clean test environment\r\n    it(\"should work like require()\", function () {\r\n        expect(rewire(\"./moduleA.js\")).to.be(require(\"./moduleA.js\"));\r\n        cleanRequireCache();\r\n        expect(rewire(\"../testModules/moduleA.js\")).to.be(require(\"../testModules/moduleA.js\"));\r\n        cleanRequireCache();\r\n        expect(rewire(\"./moduleA.js\")).to.be(require(\"./moduleA.js\"));\r\n    });\r\n    it(\"should modify the module so it provides a __set__ - function\", function () {\r\n        expect(rewire(\"./moduleA.js\").__set__).to.be.a(Function);\r\n        expect(rewire(\"./moduleB.js\").__set__).to.be.a(Function);\r\n    });\r\n    it(\"should modify the module so it provides a __get__ - function\", function () {\r\n        expect(rewire(\"./moduleA.js\").__get__).to.be.a(Function);\r\n        expect(rewire(\"./moduleB.js\").__get__).to.be.a(Function);\r\n    });\r\n    it(\"should not influence other modules\", function () {\r\n        var rewiredModuleA = rewire(\"./moduleA.js\");\r\n\r\n        expect(require(\"./someOtherModule.js\").__set__).to.be(undefined);\r\n        expect(require(\"./someOtherModule.js\").__get__).to.be(undefined);\r\n        expect(require(\"fs\").__set__).to.be(undefined);\r\n        expect(require(\"fs\").__get__).to.be(undefined);\r\n    });\r\n    it(\"should not influence global objects by default\", function () {\r\n        // This should throw no exception\r\n        rewire(\"./moduleA.js\").checkSomeGlobals();\r\n        rewire(\"./moduleB.js\").checkSomeGlobals();\r\n    });\r\n    it(\"should provide the ability to set private vars\", function () {\r\n        var rewiredModuleA = rewire(\"./moduleA.js\"),\r\n            newObj = {};\r\n\r\n        expect(rewiredModuleA.getMyNumber()).to.be(0);\r\n        rewiredModuleA.__set__(\"myNumber\", 2);\r\n        expect(rewiredModuleA.getMyNumber()).to.be(2);\r\n        rewiredModuleA.__set__(\"myObj\", newObj);\r\n        expect(rewiredModuleA.getMyObj()).to.be(newObj);\r\n        rewiredModuleA.__set__(\"env\", \"ENVENV\");\r\n    });\r\n    it(\"should provide the ability to get private vars\", function () {\r\n        var rewiredModuleA = rewire(\"./moduleA.js\");\r\n\r\n        expect(rewiredModuleA.__get__(\"myNumber\")).to.be(rewiredModuleA.getMyNumber());\r\n        expect(rewiredModuleA.__get__(\"myObj\")).to.be(rewiredModuleA.getMyObj());\r\n    });\r\n    it(\"should provide the ability to inject mocks\", function (done) {\r\n        var rewiredModuleA = rewire(\"./moduleA.js\"),\r\n            mockedFs = {\r\n                readFileSync: function (file) {\r\n                    expect(file).to.be(\"bla.txt\");\r\n                    done();\r\n                }\r\n            };\r\n\r\n        rewiredModuleA.__set__(\"fs\", mockedFs);\r\n        rewiredModuleA.readFileSync();\r\n    });\r\n    it(\"should not influence other modules when injecting mocks\", function () {\r\n        var rewiredModuleA = rewire(\"./moduleA.js\"),\r\n            someOtherModule,\r\n            mockedFs = {};\r\n\r\n        rewiredModuleA.__set__(\"fs\", mockedFs);\r\n        someOtherModule = require(\"./someOtherModule.js\");\r\n        expect(someOtherModule.fs).not.to.be(mockedFs);\r\n    });\r\n    it(\"should provide the ability to mock global objects just within the module\", function () {\r\n        var rewiredModuleA = rewire(\"./moduleA.js\"),\r\n            rewiredModuleB = rewire(\"./moduleB.js\"),\r\n            consoleMock = {},\r\n            newFilename = \"myFile.js\";\r\n\r\n        rewiredModuleA.__set__({\r\n            console: consoleMock\r\n        });\r\n        rewiredModuleA.__set__(\"__filename\", newFilename);\r\n        rewiredModuleB.__set__({\r\n            console: consoleMock\r\n        });\r\n        rewiredModuleB.__set__(\"__filename\", newFilename);\r\n        expect(rewiredModuleA.getConsole()).to.be(consoleMock);\r\n        expect(rewiredModuleB.getConsole()).to.be(consoleMock);\r\n        expect(console).not.to.be(consoleMock);\r\n        expect(rewiredModuleA.getFilename()).to.be(newFilename);\r\n        expect(rewiredModuleB.getFilename()).to.be(newFilename);\r\n    });\r\n    it(\"should cache the rewired module\", function () {\r\n        var rewired;\r\n\r\n        rewired = rewire(\"./someOtherModule.js\");\r\n        expect(require(\"./moduleA.js\").someOtherModule).to.be(rewired);\r\n        cleanRequireCache();\r\n        rewired = rewire(\"./someOtherModule.js\", true);\r\n        expect(require(\"./moduleA.js\").someOtherModule).to.be(rewired);\r\n    });\r\n    it(\"should not cache the rewired module on demand\", function () {\r\n        var rewired;\r\n\r\n        rewired = rewire(\"./someOtherModule.js\", false);\r\n        expect(require(\"./moduleA.js\").someOtherModule).not.to.be(rewired);\r\n    });\r\n    it(\"should not influence the original node require if nothing has been required within the rewired module\", function () {\r\n        rewire(\"./emptyModule.js\"); // nothing happens here because emptyModule doesn't require anything\r\n        expect(require(\"./moduleA.js\").__set__).to.be(undefined); // if restoring the original node require didn't worked, the module would have a setter\r\n    });\r\n    it(\"subsequent calls of rewire should always return a new instance\", function () {\r\n        expect(rewire(\"./moduleA.js\")).not.to.be(rewire(\"./moduleA.js\"));\r\n    });\r\n    it(\"should preserve the strict mode\", function () {\r\n        var strictModule = rewire(\"./strictModule.js\");\r\n\r\n        expect(function () {\r\n            strictModule.doSomethingUnstrict();\r\n        }).to.throwException(checkForTypeError);\r\n    });\r\n    describe(\"#reset\", function () {\r\n        it(\"should remove all rewired modules from cache\", function () {\r\n            var rewiredModuleA = rewire(\"./moduleA.js\"),\r\n                rewiredModuleB = rewire(\"./moduleB.js\");\r\n\r\n            expect(require(\"./moduleA.js\")).to.be(rewiredModuleA);\r\n            expect(require(\"./moduleB.js\")).to.be(rewiredModuleB);\r\n            rewire.reset();\r\n            expect(require(\"./moduleA.js\")).not.to.be(rewiredModuleA);\r\n            expect(require(\"./moduleB.js\")).not.to.be(rewiredModuleB);\r\n        });\r\n    });\r\n});\n//@ sourceURL=/test/testModules/sharedTestCases.js"));

require.define("/node_modules/expect.js/package.json",Function(['require','module','exports','__dirname','__filename','process'],"module.exports = {\"main\":\"./expect\"}\n//@ sourceURL=/node_modules/expect.js/package.json"));

require.define("/node_modules/expect.js/expect.js",Function(['require','module','exports','__dirname','__filename','process'],"/* this line was injected by rewire() */var global = window; var ArrayBuffer = global.ArrayBuffer; var Int8Array = global.Int8Array; var Uint8Array = global.Uint8Array; var Int16Array = global.Int16Array; var Uint16Array = global.Uint16Array; var Int32Array = global.Int32Array; var Uint32Array = global.Uint32Array; var Float32Array = global.Float32Array; var Float64Array = global.Float64Array; var DataView = global.DataView; var process = global.process; var GLOBAL = global.GLOBAL; var root = global.root; var Buffer = global.Buffer; var setTimeout = global.setTimeout; var setInterval = global.setInterval; var clearTimeout = global.clearTimeout; var clearInterval = global.clearInterval; var console = global.console; var xdescribe = global.xdescribe; var xit = global.xit; var before = global.before; var after = global.after; var beforeEach = global.beforeEach; var afterEach = global.afterEach; var describe = global.describe; var it = global.it; require(\"rewire\").register(__filename, function __set__() { arguments.varName = arguments[0]; arguments.varValue = arguments[1]; arguments.src = \"\"; arguments.checkExistsSrc = function (varName, varValue) { return \"if (typeof \" + varName + \" === 'undefined') { throw new ReferenceError('Cannot __set__(\" + varName + \", \" + varValue + \"): \" + varName + \" is not declared within the module.'); } \"; }; if (typeof arguments[0] === \"object\") { arguments.env = arguments.varName; if (!arguments.env || Array.isArray(arguments.env)) { throw new TypeError(\"__set__ expects an object as env\"); } for (arguments.key in arguments.env) { if (arguments.env.hasOwnProperty(arguments.key)) { arguments.src += arguments.checkExistsSrc(arguments.key) + arguments.key + \" = arguments.env.\" + arguments.key + \";\"; } } } else if (typeof arguments.varName === \"string\") { if (!arguments.varName) { throw new TypeError(\"__set__ expects a non-empty string as a variable name\"); } arguments.src = arguments.checkExistsSrc(arguments.varName, arguments.varValue) + arguments.varName + \" = arguments.varValue;\"; } else { throw new TypeError(\"__set__ expects an environment object or a non-empty string as a variable name\"); } eval(arguments.src); }, function __get__() { arguments.varName = arguments[0]; if (typeof arguments.varName !== \"string\" || arguments.varName.length === 0) { throw new TypeError(\"__get__ expects a non-empty string\"); } return eval(arguments.varName); });require = window.browserifyRequire.getProxy(require, __filename);\n\n(function (global, module) {\n\n  if ('undefined' == typeof module) {\n    var module = { exports: {} }\n      , exports = module.exports\n  }\n\n  /**\n   * Exports.\n   */\n\n  module.exports = expect;\n  expect.Assertion = Assertion;\n\n  /**\n   * Exports version.\n   */\n\n  expect.version = '0.1.2';\n\n  /**\n   * Possible assertion flags.\n   */\n\n  var flags = {\n      not: ['to', 'be', 'have', 'include', 'only']\n    , to: ['be', 'have', 'include', 'only', 'not']\n    , only: ['have']\n    , have: ['own']\n    , be: ['an']\n  };\n\n  function expect (obj) {\n    return new Assertion(obj);\n  }\n\n  /**\n   * Constructor\n   *\n   * @api private\n   */\n\n  function Assertion (obj, flag, parent) {\n    this.obj = obj;\n    this.flags = {};\n\n    if (undefined != parent) {\n      this.flags[flag] = true;\n\n      for (var i in parent.flags) {\n        if (parent.flags.hasOwnProperty(i)) {\n          this.flags[i] = true;\n        }\n      }\n    }\n\n    var $flags = flag ? flags[flag] : keys(flags)\n      , self = this\n\n    if ($flags) {\n      for (var i = 0, l = $flags.length; i < l; i++) {\n        // avoid recursion\n        if (this.flags[$flags[i]]) continue;\n\n        var name = $flags[i]\n          , assertion = new Assertion(this.obj, name, this)\n  \n        if ('function' == typeof Assertion.prototype[name]) {\n          // clone the function, make sure we dont touch the prot reference\n          var old = this[name];\n          this[name] = function () {\n            return old.apply(self, arguments);\n          }\n\n          for (var fn in Assertion.prototype) {\n            if (Assertion.prototype.hasOwnProperty(fn) && fn != name) {\n              this[name][fn] = bind(assertion[fn], assertion);\n            }\n          }\n        } else {\n          this[name] = assertion;\n        }\n      }\n    }\n  };\n\n  /**\n   * Performs an assertion\n   *\n   * @api private\n   */\n\n  Assertion.prototype.assert = function (truth, msg, error) {\n    var msg = this.flags.not ? error : msg\n      , ok = this.flags.not ? !truth : truth;\n\n    if (!ok) {\n      throw new Error(msg);\n    }\n\n    this.and = new Assertion(this.obj);\n  };\n\n  /**\n   * Check if the value is truthy\n   *\n   * @api public\n   */\n\n  Assertion.prototype.ok = function () {\n    this.assert(\n        !!this.obj\n      , 'expected ' + i(this.obj) + ' to be truthy'\n      , 'expected ' + i(this.obj) + ' to be falsy');\n  };\n\n  /**\n   * Assert that the function throws.\n   *\n   * @param {Function|RegExp} callback, or regexp to match error string against\n   * @api public\n   */\n\n  Assertion.prototype.throwError =\n  Assertion.prototype.throwException = function (fn) {\n    expect(this.obj).to.be.a('function');\n\n    var thrown = false\n      , not = this.flags.not\n\n    try {\n      this.obj();\n    } catch (e) {\n      if ('function' == typeof fn) {\n        fn(e);\n      } else if ('object' == typeof fn) {\n        var subject = 'string' == typeof e ? e : e.message;\n        if (not) {\n          expect(subject).to.not.match(fn);\n        } else {\n          expect(subject).to.match(fn);\n        }\n      }\n      thrown = true;\n    }\n\n    if ('object' == typeof fn && not) {\n      // in the presence of a matcher, ensure the `not` only applies to\n      // the matching.\n      this.flags.not = false; \n    }\n\n    var name = this.obj.name || 'fn';\n    this.assert(\n        thrown\n      , 'expected ' + name + ' to throw an exception'\n      , 'expected ' + name + ' not to throw an exception');\n  };\n\n  /**\n   * Checks if the array is empty.\n   *\n   * @api public\n   */\n\n  Assertion.prototype.empty = function () {\n    var expectation;\n\n    if ('object' == typeof this.obj && null !== this.obj && !isArray(this.obj)) {\n      if ('number' == typeof this.obj.length) {\n        expectation = !this.obj.length;\n      } else {\n        expectation = !keys(this.obj).length;\n      }\n    } else {\n      if ('string' != typeof this.obj) {\n        expect(this.obj).to.be.an('object');\n      }\n\n      expect(this.obj).to.have.property('length');\n      expectation = !this.obj.length;\n    }\n\n    this.assert(\n        expectation\n      , 'expected ' + i(this.obj) + ' to be empty'\n      , 'expected ' + i(this.obj) + ' to not be empty');\n    return this;\n  };\n\n  /**\n   * Checks if the obj exactly equals another.\n   *\n   * @api public\n   */\n\n  Assertion.prototype.be =\n  Assertion.prototype.equal = function (obj) {\n    this.assert(\n        obj === this.obj\n      , 'expected ' + i(this.obj) + ' to equal ' + i(obj)\n      , 'expected ' + i(this.obj) + ' to not equal ' + i(obj));\n    return this;\n  };\n\n  /**\n   * Checks if the obj sortof equals another.\n   *\n   * @api public\n   */\n\n  Assertion.prototype.eql = function (obj) {\n    this.assert(\n        expect.eql(obj, this.obj)\n      , 'expected ' + i(this.obj) + ' to sort of equal ' + i(obj)\n      , 'expected ' + i(this.obj) + ' to sort of not equal ' + i(obj));\n    return this;\n  };\n\n  /**\n   * Assert within start to finish (inclusive). \n   *\n   * @param {Number} start\n   * @param {Number} finish\n   * @api public\n   */\n\n  Assertion.prototype.within = function (start, finish) {\n    var range = start + '..' + finish;\n    this.assert(\n        this.obj >= start && this.obj <= finish\n      , 'expected ' + i(this.obj) + ' to be within ' + range\n      , 'expected ' + i(this.obj) + ' to not be within ' + range);\n    return this;\n  };\n\n  /**\n   * Assert typeof / instance of\n   *\n   * @api public\n   */\n\n  Assertion.prototype.a =\n  Assertion.prototype.an = function (type) {\n    if ('string' == typeof type) {\n      // proper english in error msg\n      var n = /^[aeiou]/.test(type) ? 'n' : '';\n\n      // typeof with support for 'array'\n      this.assert(\n          'array' == type ? isArray(this.obj) :\n            'object' == type\n              ? 'object' == typeof this.obj && null !== this.obj\n              : type == typeof this.obj\n        , 'expected ' + i(this.obj) + ' to be a' + n + ' ' + type\n        , 'expected ' + i(this.obj) + ' not to be a' + n + ' ' + type);\n    } else {\n      // instanceof\n      var name = type.name || 'supplied constructor';\n      this.assert(\n          this.obj instanceof type\n        , 'expected ' + i(this.obj) + ' to be an instance of ' + name\n        , 'expected ' + i(this.obj) + ' not to be an instance of ' + name);\n    }\n\n    return this;\n  };\n\n  /**\n   * Assert numeric value above _n_.\n   *\n   * @param {Number} n\n   * @api public\n   */\n\n  Assertion.prototype.greaterThan =\n  Assertion.prototype.above = function (n) {\n    this.assert(\n        this.obj > n\n      , 'expected ' + i(this.obj) + ' to be above ' + n\n      , 'expected ' + i(this.obj) + ' to be below ' + n);\n    return this;\n  };\n\n  /**\n   * Assert numeric value below _n_.\n   *\n   * @param {Number} n\n   * @api public\n   */\n\n  Assertion.prototype.lessThan =\n  Assertion.prototype.below = function (n) {\n    this.assert(\n        this.obj < n\n      , 'expected ' + i(this.obj) + ' to be below ' + n\n      , 'expected ' + i(this.obj) + ' to be above ' + n);\n    return this;\n  };\n  \n  /**\n   * Assert string value matches _regexp_.\n   *\n   * @param {RegExp} regexp\n   * @api public\n   */\n\n  Assertion.prototype.match = function (regexp) {\n    this.assert(\n        regexp.exec(this.obj)\n      , 'expected ' + i(this.obj) + ' to match ' + regexp\n      , 'expected ' + i(this.obj) + ' not to match ' + regexp);\n    return this;\n  };\n\n  /**\n   * Assert property \"length\" exists and has value of _n_.\n   *\n   * @param {Number} n\n   * @api public\n   */\n\n  Assertion.prototype.length = function (n) {\n    expect(this.obj).to.have.property('length');\n    var len = this.obj.length;\n    this.assert(\n        n == len\n      , 'expected ' + i(this.obj) + ' to have a length of ' + n + ' but got ' + len\n      , 'expected ' + i(this.obj) + ' to not have a length of ' + len);\n    return this;\n  };\n\n  /**\n   * Assert property _name_ exists, with optional _val_.\n   *\n   * @param {String} name\n   * @param {Mixed} val\n   * @api public\n   */\n\n  Assertion.prototype.property = function (name, val) {\n    if (this.flags.own) {\n      this.assert(\n          Object.prototype.hasOwnProperty.call(this.obj, name)\n        , 'expected ' + i(this.obj) + ' to have own property ' + i(name)\n        , 'expected ' + i(this.obj) + ' to not have own property ' + i(name));\n      return this;\n    }\n\n    if (this.flags.not && undefined !== val) {\n      if (undefined === this.obj[name]) {\n        throw new Error(i(this.obj) + ' has no property ' + i(name));\n      }\n    } else {\n      var hasProp;\n      try {\n        hasProp = name in this.obj\n      } catch (e) {\n        hasProp = undefined !== this.obj[name]\n      }\n      \n      this.assert(\n          hasProp\n        , 'expected ' + i(this.obj) + ' to have a property ' + i(name)\n        , 'expected ' + i(this.obj) + ' to not have a property ' + i(name));\n    }\n    \n    if (undefined !== val) {\n      this.assert(\n          val === this.obj[name]\n        , 'expected ' + i(this.obj) + ' to have a property ' + i(name)\n          + ' of ' + i(val) + ', but got ' + i(this.obj[name])\n        , 'expected ' + i(this.obj) + ' to not have a property ' + i(name)\n          + ' of ' + i(val));\n    }\n\n    this.obj = this.obj[name];\n    return this;\n  };\n\n  /**\n   * Assert that the array contains _obj_ or string contains _obj_.\n   *\n   * @param {Mixed} obj|string\n   * @api public\n   */\n\n  Assertion.prototype.string =\n  Assertion.prototype.contain = function (obj) {\n    if ('string' == typeof this.obj) {\n      this.assert(\n          ~this.obj.indexOf(obj)\n        , 'expected ' + i(this.obj) + ' to contain ' + i(obj)\n        , 'expected ' + i(this.obj) + ' to not contain ' + i(obj));\n    } else {\n      this.assert(\n          ~indexOf(this.obj, obj)\n        , 'expected ' + i(this.obj) + ' to contain ' + i(obj)\n        , 'expected ' + i(this.obj) + ' to not contain ' + i(obj));\n    }\n    return this;\n  };\n\n  /**\n   * Assert exact keys or inclusion of keys by using\n   * the `.own` modifier.\n   *\n   * @param {Array|String ...} keys\n   * @api public\n   */\n\n  Assertion.prototype.key =\n  Assertion.prototype.keys = function ($keys) {\n    var str\n      , ok = true;\n\n    $keys = isArray($keys)\n      ? $keys\n      : Array.prototype.slice.call(arguments);\n\n    if (!$keys.length) throw new Error('keys required');\n\n    var actual = keys(this.obj)\n      , len = $keys.length;\n\n    // Inclusion\n    ok = every($keys, function (key) {\n      return ~indexOf(actual, key);\n    });\n\n    // Strict\n    if (!this.flags.not && this.flags.only) {\n      ok = ok && $keys.length == actual.length;\n    }\n\n    // Key string\n    if (len > 1) {\n      $keys = map($keys, function (key) {\n        return i(key);\n      });\n      var last = $keys.pop();\n      str = $keys.join(', ') + ', and ' + last;\n    } else {\n      str = i($keys[0]);\n    }\n\n    // Form\n    str = (len > 1 ? 'keys ' : 'key ') + str;\n\n    // Have / include\n    str = (!this.flags.only ? 'include ' : 'only have ') + str;\n\n    // Assertion\n    this.assert(\n        ok\n      , 'expected ' + i(this.obj) + ' to ' + str\n      , 'expected ' + i(this.obj) + ' to not ' + str);\n\n    return this;\n  };\n\n  /**\n   * Function bind implementation.\n   */\n\n  function bind (fn, scope) {\n    return function () {\n      return fn.apply(scope, arguments);\n    }\n  }\n\n  /**\n   * Array every compatibility\n   *\n   * @see bit.ly/5Fq1N2\n   * @api public\n   */\n\n  function every (arr, fn, thisObj) {\n    var scope = thisObj || global;\n    for (var i = 0, j = arr.length; i < j; ++i) {\n      if (!fn.call(scope, arr[i], i, arr)) {\n        return false;\n      }\n    }\n    return true;\n  };\n\n  /**\n   * Array indexOf compatibility.\n   *\n   * @see bit.ly/a5Dxa2\n   * @api public\n   */\n\n  function indexOf (arr, o, i) {\n    if (Array.prototype.indexOf) {\n      return Array.prototype.indexOf.call(arr, o, i);\n    }\n\n    if (arr.length === undefined) {\n      return -1;\n    }\n\n    for (var j = arr.length, i = i < 0 ? i + j < 0 ? 0 : i + j : i || 0\n        ; i < j && arr[i] !== o; i++);\n\n    return j <= i ? -1 : i;\n  };\n\n  /**\n   * Inspects an object.\n   *\n   * @see taken from node.js `util` module (copyright Joyent, MIT license)\n   * @api private\n   */\n\n  function i (obj, showHidden, depth) {\n    var seen = [];\n\n    function stylize (str) {\n      return str;\n    };\n\n    function format (value, recurseTimes) {\n      // Provide a hook for user-specified inspect functions.\n      // Check that value is an object with an inspect function on it\n      if (value && typeof value.inspect === 'function' &&\n          // Filter out the util module, it's inspect function is special\n          value !== exports &&\n          // Also filter out any prototype objects using the circular check.\n          !(value.constructor && value.constructor.prototype === value)) {\n        return value.inspect(recurseTimes);\n      }\n\n      // Primitive types cannot have properties\n      switch (typeof value) {\n        case 'undefined':\n          return stylize('undefined', 'undefined');\n\n        case 'string':\n          var simple = '\\'' + json.stringify(value).replace(/^\"|\"$/g, '')\n                                                   .replace(/'/g, \"\\\\'\")\n                                                   .replace(/\\\\\"/g, '\"') + '\\'';\n          return stylize(simple, 'string');\n\n        case 'number':\n          return stylize('' + value, 'number');\n\n        case 'boolean':\n          return stylize('' + value, 'boolean');\n      }\n      // For some reason typeof null is \"object\", so special case here.\n      if (value === null) {\n        return stylize('null', 'null');\n      }\n\n      // Look up the keys of the object.\n      var visible_keys = keys(value);\n      var $keys = showHidden ? Object.getOwnPropertyNames(value) : visible_keys;\n\n      // Functions without properties can be shortcutted.\n      if (typeof value === 'function' && $keys.length === 0) {\n        if (isRegExp(value)) {\n          return stylize('' + value, 'regexp');\n        } else {\n          var name = value.name ? ': ' + value.name : '';\n          return stylize('[Function' + name + ']', 'special');\n        }\n      }\n\n      // Dates without properties can be shortcutted\n      if (isDate(value) && $keys.length === 0) {\n        return stylize(value.toUTCString(), 'date');\n      }\n\n      var base, type, braces;\n      // Determine the object type\n      if (isArray(value)) {\n        type = 'Array';\n        braces = ['[', ']'];\n      } else {\n        type = 'Object';\n        braces = ['{', '}'];\n      }\n\n      // Make functions say that they are functions\n      if (typeof value === 'function') {\n        var n = value.name ? ': ' + value.name : '';\n        base = (isRegExp(value)) ? ' ' + value : ' [Function' + n + ']';\n      } else {\n        base = '';\n      }\n\n      // Make dates with properties first say the date\n      if (isDate(value)) {\n        base = ' ' + value.toUTCString();\n      }\n\n      if ($keys.length === 0) {\n        return braces[0] + base + braces[1];\n      }\n\n      if (recurseTimes < 0) {\n        if (isRegExp(value)) {\n          return stylize('' + value, 'regexp');\n        } else {\n          return stylize('[Object]', 'special');\n        }\n      }\n\n      seen.push(value);\n\n      var output = map($keys, function (key) {\n        var name, str;\n        if (value.__lookupGetter__) {\n          if (value.__lookupGetter__(key)) {\n            if (value.__lookupSetter__(key)) {\n              str = stylize('[Getter/Setter]', 'special');\n            } else {\n              str = stylize('[Getter]', 'special');\n            }\n          } else {\n            if (value.__lookupSetter__(key)) {\n              str = stylize('[Setter]', 'special');\n            }\n          }\n        }\n        if (indexOf(visible_keys, key) < 0) {\n          name = '[' + key + ']';\n        }\n        if (!str) {\n          if (indexOf(seen, value[key]) < 0) {\n            if (recurseTimes === null) {\n              str = format(value[key]);\n            } else {\n              str = format(value[key], recurseTimes - 1);\n            }\n            if (str.indexOf('\\n') > -1) {\n              if (isArray(value)) {\n                str = map(str.split('\\n'), function (line) {\n                  return '  ' + line;\n                }).join('\\n').substr(2);\n              } else {\n                str = '\\n' + map(str.split('\\n'), function (line) {\n                  return '   ' + line;\n                }).join('\\n');\n              }\n            }\n          } else {\n            str = stylize('[Circular]', 'special');\n          }\n        }\n        if (typeof name === 'undefined') {\n          if (type === 'Array' && key.match(/^\\d+$/)) {\n            return str;\n          }\n          name = json.stringify('' + key);\n          if (name.match(/^\"([a-zA-Z_][a-zA-Z_0-9]*)\"$/)) {\n            name = name.substr(1, name.length - 2);\n            name = stylize(name, 'name');\n          } else {\n            name = name.replace(/'/g, \"\\\\'\")\n                       .replace(/\\\\\"/g, '\"')\n                       .replace(/(^\"|\"$)/g, \"'\");\n            name = stylize(name, 'string');\n          }\n        }\n\n        return name + ': ' + str;\n      });\n\n      seen.pop();\n\n      var numLinesEst = 0;\n      var length = reduce(output, function (prev, cur) {\n        numLinesEst++;\n        if (indexOf(cur, '\\n') >= 0) numLinesEst++;\n        return prev + cur.length + 1;\n      }, 0);\n\n      if (length > 50) {\n        output = braces[0] +\n                 (base === '' ? '' : base + '\\n ') +\n                 ' ' +\n                 output.join(',\\n  ') +\n                 ' ' +\n                 braces[1];\n\n      } else {\n        output = braces[0] + base + ' ' + output.join(', ') + ' ' + braces[1];\n      }\n\n      return output;\n    }\n    return format(obj, (typeof depth === 'undefined' ? 2 : depth));\n  };\n\n  function isArray (ar) {\n    return Object.prototype.toString.call(ar) == '[object Array]';\n  };\n\n  function isRegExp(re) {\n    var s = '' + re;\n    return re instanceof RegExp || // easy case\n           // duck-type for context-switching evalcx case\n           typeof(re) === 'function' &&\n           re.constructor.name === 'RegExp' &&\n           re.compile &&\n           re.test &&\n           re.exec &&\n           s.match(/^\\/.*\\/[gim]{0,3}$/);\n  };\n\n  function isDate(d) {\n    if (d instanceof Date) return true;\n    return false;\n  };\n\n  function keys (obj) {\n    if (Object.keys) {\n      return Object.keys(obj);\n    }\n\n    var keys = [];\n\n    for (var i in obj) {\n      if (Object.prototype.hasOwnProperty.call(obj, i)) {\n        keys.push(i);\n      }\n    }\n\n    return keys;\n  }\n\n  function map (arr, mapper, that) {\n    if (Array.prototype.map) {\n      return Array.prototype.map.call(arr, mapper, that);\n    }\n\n    var other= new Array(arr.length);\n\n    for (var i= 0, n = arr.length; i<n; i++)\n      if (i in arr)\n        other[i] = mapper.call(that, arr[i], i, arr);\n\n    return other;\n  };\n\n  function reduce (arr, fun) {\n    if (Array.prototype.reduce) {\n      return Array.prototype.reduce.apply(\n          arr\n        , Array.prototype.slice.call(arguments, 1)\n      );\n    }\n\n    var len = +this.length;\n\n    if (typeof fun !== \"function\")\n      throw new TypeError();\n\n    // no value to return if no initial value and an empty array\n    if (len === 0 && arguments.length === 1)\n      throw new TypeError();\n\n    var i = 0;\n    if (arguments.length >= 2) {\n      var rv = arguments[1];\n    } else {\n      do {\n        if (i in this) {\n          rv = this[i++];\n          break;\n        }\n\n        // if array contains no values, no initial value to return\n        if (++i >= len)\n          throw new TypeError();\n      } while (true);\n    }\n\n    for (; i < len; i++) {\n      if (i in this)\n        rv = fun.call(null, rv, this[i], i, this);\n    }\n\n    return rv;\n  };\n\n  /**\n   * Asserts deep equality\n   *\n   * @see taken from node.js `assert` module (copyright Joyent, MIT license)\n   * @api private\n   */\n\n  expect.eql = function eql (actual, expected) {\n    // 7.1. All identical values are equivalent, as determined by ===.\n    if (actual === expected) { \n      return true;\n    } else if ('undefined' != typeof Buffer \n        && Buffer.isBuffer(actual) && Buffer.isBuffer(expected)) {\n      if (actual.length != expected.length) return false;\n\n      for (var i = 0; i < actual.length; i++) {\n        if (actual[i] !== expected[i]) return false;\n      }\n\n      return true;\n\n    // 7.2. If the expected value is a Date object, the actual value is\n    // equivalent if it is also a Date object that refers to the same time.\n    } else if (actual instanceof Date && expected instanceof Date) {\n      return actual.getTime() === expected.getTime();\n\n    // 7.3. Other pairs that do not both pass typeof value == \"object\",\n    // equivalence is determined by ==.\n    } else if (typeof actual != 'object' && typeof expected != 'object') {\n      return actual == expected;\n\n    // 7.4. For all other Object pairs, including Array objects, equivalence is\n    // determined by having the same number of owned properties (as verified\n    // with Object.prototype.hasOwnProperty.call), the same set of keys\n    // (although not necessarily the same order), equivalent values for every\n    // corresponding key, and an identical \"prototype\" property. Note: this\n    // accounts for both named and indexed properties on Arrays.\n    } else {\n      return objEquiv(actual, expected);\n    }\n  }\n\n  function isUndefinedOrNull (value) {\n    return value === null || value === undefined;\n  }\n\n  function isArguments (object) {\n    return Object.prototype.toString.call(object) == '[object Arguments]';\n  }\n\n  function objEquiv (a, b) {\n    if (isUndefinedOrNull(a) || isUndefinedOrNull(b))\n      return false;\n    // an identical \"prototype\" property.\n    if (a.prototype !== b.prototype) return false;\n    //~~~I've managed to break Object.keys through screwy arguments passing.\n    //   Converting to array solves the problem.\n    if (isArguments(a)) {\n      if (!isArguments(b)) {\n        return false;\n      }\n      a = pSlice.call(a);\n      b = pSlice.call(b);\n      return expect.eql(a, b);\n    }\n    try{\n      var ka = keys(a),\n        kb = keys(b),\n        key, i;\n    } catch (e) {//happens when one is a string literal and the other isn't\n      return false;\n    }\n    // having the same number of owned properties (keys incorporates hasOwnProperty)\n    if (ka.length != kb.length)\n      return false;\n    //the same set of keys (although not necessarily the same order),\n    ka.sort();\n    kb.sort();\n    //~~~cheap key test\n    for (i = ka.length - 1; i >= 0; i--) {\n      if (ka[i] != kb[i])\n        return false;\n    }\n    //equivalent values for every corresponding key, and\n    //~~~possibly expensive deep test\n    for (i = ka.length - 1; i >= 0; i--) {\n      key = ka[i];\n      if (!expect.eql(a[key], b[key]))\n         return false;\n    }\n    return true;\n  }\n\n  var json = (function () {\n    \"use strict\";\n\n    if ('object' == typeof JSON && JSON.parse && JSON.stringify) {\n      return {\n          parse: nativeJSON.parse\n        , stringify: nativeJSON.stringify\n      }\n    }\n\n    var JSON = {};\n\n    function f(n) {\n        // Format integers to have at least two digits.\n        return n < 10 ? '0' + n : n;\n    }\n\n    function date(d, key) {\n      return isFinite(d.valueOf()) ?\n          d.getUTCFullYear()     + '-' +\n          f(d.getUTCMonth() + 1) + '-' +\n          f(d.getUTCDate())      + 'T' +\n          f(d.getUTCHours())     + ':' +\n          f(d.getUTCMinutes())   + ':' +\n          f(d.getUTCSeconds())   + 'Z' : null;\n    };\n\n    var cx = /[\\u0000\\u00ad\\u0600-\\u0604\\u070f\\u17b4\\u17b5\\u200c-\\u200f\\u2028-\\u202f\\u2060-\\u206f\\ufeff\\ufff0-\\uffff]/g,\n        escapable = /[\\\\\\\"\\x00-\\x1f\\x7f-\\x9f\\u00ad\\u0600-\\u0604\\u070f\\u17b4\\u17b5\\u200c-\\u200f\\u2028-\\u202f\\u2060-\\u206f\\ufeff\\ufff0-\\uffff]/g,\n        gap,\n        indent,\n        meta = {    // table of character substitutions\n            '\\b': '\\\\b',\n            '\\t': '\\\\t',\n            '\\n': '\\\\n',\n            '\\f': '\\\\f',\n            '\\r': '\\\\r',\n            '\"' : '\\\\\"',\n            '\\\\': '\\\\\\\\'\n        },\n        rep;\n\n\n    function quote(string) {\n\n  // If the string contains no control characters, no quote characters, and no\n  // backslash characters, then we can safely slap some quotes around it.\n  // Otherwise we must also replace the offending characters with safe escape\n  // sequences.\n\n        escapable.lastIndex = 0;\n        return escapable.test(string) ? '\"' + string.replace(escapable, function (a) {\n            var c = meta[a];\n            return typeof c === 'string' ? c :\n                '\\\\u' + ('0000' + a.charCodeAt(0).toString(16)).slice(-4);\n        }) + '\"' : '\"' + string + '\"';\n    }\n\n\n    function str(key, holder) {\n\n  // Produce a string from holder[key].\n\n        var i,          // The loop counter.\n            k,          // The member key.\n            v,          // The member value.\n            length,\n            mind = gap,\n            partial,\n            value = holder[key];\n\n  // If the value has a toJSON method, call it to obtain a replacement value.\n\n        if (value instanceof Date) {\n            value = date(key);\n        }\n\n  // If we were called with a replacer function, then call the replacer to\n  // obtain a replacement value.\n\n        if (typeof rep === 'function') {\n            value = rep.call(holder, key, value);\n        }\n\n  // What happens next depends on the value's type.\n\n        switch (typeof value) {\n        case 'string':\n            return quote(value);\n\n        case 'number':\n\n  // JSON numbers must be finite. Encode non-finite numbers as null.\n\n            return isFinite(value) ? String(value) : 'null';\n\n        case 'boolean':\n        case 'null':\n\n  // If the value is a boolean or null, convert it to a string. Note:\n  // typeof null does not produce 'null'. The case is included here in\n  // the remote chance that this gets fixed someday.\n\n            return String(value);\n\n  // If the type is 'object', we might be dealing with an object or an array or\n  // null.\n\n        case 'object':\n\n  // Due to a specification blunder in ECMAScript, typeof null is 'object',\n  // so watch out for that case.\n\n            if (!value) {\n                return 'null';\n            }\n\n  // Make an array to hold the partial results of stringifying this object value.\n\n            gap += indent;\n            partial = [];\n\n  // Is the value an array?\n\n            if (Object.prototype.toString.apply(value) === '[object Array]') {\n\n  // The value is an array. Stringify every element. Use null as a placeholder\n  // for non-JSON values.\n\n                length = value.length;\n                for (i = 0; i < length; i += 1) {\n                    partial[i] = str(i, value) || 'null';\n                }\n\n  // Join all of the elements together, separated with commas, and wrap them in\n  // brackets.\n\n                v = partial.length === 0 ? '[]' : gap ?\n                    '[\\n' + gap + partial.join(',\\n' + gap) + '\\n' + mind + ']' :\n                    '[' + partial.join(',') + ']';\n                gap = mind;\n                return v;\n            }\n\n  // If the replacer is an array, use it to select the members to be stringified.\n\n            if (rep && typeof rep === 'object') {\n                length = rep.length;\n                for (i = 0; i < length; i += 1) {\n                    if (typeof rep[i] === 'string') {\n                        k = rep[i];\n                        v = str(k, value);\n                        if (v) {\n                            partial.push(quote(k) + (gap ? ': ' : ':') + v);\n                        }\n                    }\n                }\n            } else {\n\n  // Otherwise, iterate through all of the keys in the object.\n\n                for (k in value) {\n                    if (Object.prototype.hasOwnProperty.call(value, k)) {\n                        v = str(k, value);\n                        if (v) {\n                            partial.push(quote(k) + (gap ? ': ' : ':') + v);\n                        }\n                    }\n                }\n            }\n\n  // Join all of the member texts together, separated with commas,\n  // and wrap them in braces.\n\n            v = partial.length === 0 ? '{}' : gap ?\n                '{\\n' + gap + partial.join(',\\n' + gap) + '\\n' + mind + '}' :\n                '{' + partial.join(',') + '}';\n            gap = mind;\n            return v;\n        }\n    }\n\n  // If the JSON object does not yet have a stringify method, give it one.\n\n    JSON.stringify = function (value, replacer, space) {\n\n  // The stringify method takes a value and an optional replacer, and an optional\n  // space parameter, and returns a JSON text. The replacer can be a function\n  // that can replace values, or an array of strings that will select the keys.\n  // A default replacer method can be provided. Use of the space parameter can\n  // produce text that is more easily readable.\n\n        var i;\n        gap = '';\n        indent = '';\n\n  // If the space parameter is a number, make an indent string containing that\n  // many spaces.\n\n        if (typeof space === 'number') {\n            for (i = 0; i < space; i += 1) {\n                indent += ' ';\n            }\n\n  // If the space parameter is a string, it will be used as the indent string.\n\n        } else if (typeof space === 'string') {\n            indent = space;\n        }\n\n  // If there is a replacer, it must be a function or an array.\n  // Otherwise, throw an error.\n\n        rep = replacer;\n        if (replacer && typeof replacer !== 'function' &&\n                (typeof replacer !== 'object' ||\n                typeof replacer.length !== 'number')) {\n            throw new Error('JSON.stringify');\n        }\n\n  // Make a fake root object containing our value under the key of ''.\n  // Return the result of stringifying the value.\n\n        return str('', {'': value});\n    };\n\n  // If the JSON object does not yet have a parse method, give it one.\n\n    JSON.parse = function (text, reviver) {\n    // The parse method takes a text and an optional reviver function, and returns\n    // a JavaScript value if the text is a valid JSON text.\n\n        var j;\n\n        function walk(holder, key) {\n\n    // The walk method is used to recursively walk the resulting structure so\n    // that modifications can be made.\n\n            var k, v, value = holder[key];\n            if (value && typeof value === 'object') {\n                for (k in value) {\n                    if (Object.prototype.hasOwnProperty.call(value, k)) {\n                        v = walk(value, k);\n                        if (v !== undefined) {\n                            value[k] = v;\n                        } else {\n                            delete value[k];\n                        }\n                    }\n                }\n            }\n            return reviver.call(holder, key, value);\n        }\n\n\n    // Parsing happens in four stages. In the first stage, we replace certain\n    // Unicode characters with escape sequences. JavaScript handles many characters\n    // incorrectly, either silently deleting them, or treating them as line endings.\n\n        text = String(text);\n        cx.lastIndex = 0;\n        if (cx.test(text)) {\n            text = text.replace(cx, function (a) {\n                return '\\\\u' +\n                    ('0000' + a.charCodeAt(0).toString(16)).slice(-4);\n            });\n        }\n\n    // In the second stage, we run the text against regular expressions that look\n    // for non-JSON patterns. We are especially concerned with '()' and 'new'\n    // because they can cause invocation, and '=' because it can cause mutation.\n    // But just to be safe, we want to reject all unexpected forms.\n\n    // We split the second stage into 4 regexp operations in order to work around\n    // crippling inefficiencies in IE's and Safari's regexp engines. First we\n    // replace the JSON backslash pairs with '@' (a non-JSON character). Second, we\n    // replace all simple value tokens with ']' characters. Third, we delete all\n    // open brackets that follow a colon or comma or that begin the text. Finally,\n    // we look to see that the remaining characters are only whitespace or ']' or\n    // ',' or ':' or '{' or '}'. If that is so, then the text is safe for eval.\n\n        if (/^[\\],:{}\\s]*$/\n                .test(text.replace(/\\\\(?:[\"\\\\\\/bfnrt]|u[0-9a-fA-F]{4})/g, '@')\n                    .replace(/\"[^\"\\\\\\n\\r]*\"|true|false|null|-?\\d+(?:\\.\\d*)?(?:[eE][+\\-]?\\d+)?/g, ']')\n                    .replace(/(?:^|:|,)(?:\\s*\\[)+/g, ''))) {\n\n    // In the third stage we use the eval function to compile the text into a\n    // JavaScript structure. The '{' operator is subject to a syntactic ambiguity\n    // in JavaScript: it can begin a block or an object literal. We wrap the text\n    // in parens to eliminate the ambiguity.\n\n            j = eval('(' + text + ')');\n\n    // In the optional fourth stage, we recursively walk the new structure, passing\n    // each name/value pair to a reviver function for possible transformation.\n\n            return typeof reviver === 'function' ?\n                walk({'': j}, '') : j;\n        }\n\n    // If the text is not JSON parseable, then a SyntaxError is thrown.\n\n        throw new SyntaxError('JSON.parse');\n    };\n\n    return JSON;\n  })();\n\n  if ('undefined' != typeof window) {\n    window.expect = module.exports;\n  }\n\n})(\n    this\n  , 'undefined' != typeof module ? module : {}\n  , 'undefined' != typeof exports ? exports : {}\n);\n\n//@ sourceURL=/node_modules/expect.js/expect.js"));

/**
 * This code gets injected at the end of the browserify bundle via b.append().
 */

if (typeof window.browserifyRequire !== "undefined") {
    throw new Error("Naming collision detected: window.browserifyRequire seems to be occupied.");
}

// Saves the browserifyRequire under a new name. Necessary to call the original browserifyRequire within
// a module where the variable name "require" is overridden by the module's internal require.
window.browserifyRequire = require;

/**
 * Provides a special require-proxy. Every module calls window.browserifyRequire.getProxy(require, __filename) at the
 * beginning and overrides its own require with this proxy.
 *
 * This is necessary to call rewire() with the original __filename. Thus you can use rewire() like require().
 *
 * @param {!Function} internalRequire the module's own require
 * @param {String} filename the __filename of the module
 * @return {Function} requireProxy
 */
window.browserifyRequire.getProxy = function (internalRequire, filename) {
    var rewireModule = internalRequire("rewire"),
        key;

    function rewireProxy(path, cache) {
        return rewireModule(filename, path, cache);
    }

    for (key in rewireModule) {
        if (rewireModule.hasOwnProperty(key)) {
            rewireProxy[key] = rewireModule[key];
        }
    }

    return function requireProxy(path) {
        if (path === "rewire") {
            return rewireProxy;
        } else {
            return internalRequire(path);
        }
    };
};window.onload = function () {mocha.setup("bdd");window.browserifyRequire("/test/testModules/sharedTestCases.js");mocha.run();};